import numpy as np
import cmap
from matplotlib.colors import Colormap
import matplotlib.pyplot as plt
import pytest
import mergechannels as mc


@pytest.fixture
def matplotlib_viridis_cmap() -> Colormap:
    '''
    Get the viridis colormap from matplotlib
    '''
    return plt.get_cmap('viridis')

@pytest.fixture
def cmap_mako_colormap() -> cmap.Colormap:
    '''
    Get the seaborn mako colormap from cmap
    '''
    return cmap.Colormap('seaborn:mako')

def test_apply_with_matplotlib_cmap(matplotlib_viridis_cmap: Colormap):
    '''
    Test that the color map is applied correctly with a matplotlib colormap
    '''
    x = np.ones((1, 1), dtype=np.uint8)
    rgb = mc.apply_color_map(x, matplotlib_viridis_cmap, saturation_limits=(0, 255))
    assert np.allclose(rgb, np.array([[[68, 2, 85]]]))
    x = x * 255
    rgb = mc.apply_color_map(x, matplotlib_viridis_cmap, saturation_limits=(0, 255))
    assert np.allclose(rgb, np.array([[[253, 231,  36]]]))

def test_apply_with_cmap_colormap(cmap_mako_colormap: cmap.Colormap):
    '''
    Test that the color map is applied correctly with a cmap colormap
    '''
    x = np.ones((1, 1), dtype=np.uint8)
    rgb = mc.apply_color_map(x, cmap_mako_colormap, saturation_limits=(0, 255))
    assert np.allclose(rgb, np.array([[[12, 4, 6]]]))
    x = x * 255
    rgb = mc.apply_color_map(x, cmap_mako_colormap, saturation_limits=(0, 255))
    assert np.allclose(rgb, np.array([[[222, 244, 228]]]))

def test_merge_with_both(
    matplotlib_viridis_cmap: Colormap,
    cmap_mako_colormap: cmap.Colormap,
) -> None:
    '''
    Test that the colors are merged correctly with both matplotlib and cmap colormaps
    '''
    x = np.ones((1, 1), dtype=np.uint8)
    y = np.ones((1, 1), dtype=np.uint8)
    rgb = mc.merge(
        [x, y],
        [matplotlib_viridis_cmap, cmap_mako_colormap],
        saturation_limits=[(0, 255), (0, 255)],
    )
    assert np.allclose(rgb, np.array([[[68,  4, 85]]]))
    x = x * 255
    y = y * 255
    rgb = mc.merge(
        [x, y],
        [matplotlib_viridis_cmap, cmap_mako_colormap],
        saturation_limits=[(0, 255), (0, 255)],
    )
    assert np.allclose(rgb, np.array([[[253, 244,  228]]]))


def test_apply_color_map():
    '''
    Test that the color map is applied correctly
    '''
    x = np.ones((1, 1), dtype=np.uint8)
    rgb = mc.dispatch_single_channel(x, 'betterBlue', None, (0, 255))
    assert rgb.shape == (1,1,3)
    assert rgb.dtype == np.uint8
    assert np.allclose(
        rgb,
        np.array(
            [[[0, 1, 2]]]
        )
    )
    x = np.ones((1, 1), dtype=np.uint8) * 255
    rgb = mc.dispatch_single_channel(x, 'betterBlue', None, (0, 255))
    assert np.allclose(
        rgb,
        np.array(
            [[[0, 188, 254]]]
        )
    )
    rgb2 = mc.apply_color_map(x, 'betterBlue', saturation_limits=(0, 255))
    assert np.allclose(rgb, rgb2)

def test_apply_colors_and_merge_low_sum():
    '''
    Test that the colors are merged correctly with sum blending and low values
    '''
    x = np.ones((1, 1), dtype=np.uint8)
    y = np.ones((1, 1), dtype=np.uint8)
    rgb_sum = mc.dispatch_multi_channel(
        [x, y],
        ['betterBlue', 'betterOrange'],
        [None, None],
        'sum',
        [(0, 255), (0, 255)],
    )
    # blue = [0, 1, 2,]
    # orange = [1, 1, 0]
    assert np.allclose(
        rgb_sum,
        np.array(
            [[[1, 2, 2]]]
        )
    )
    rgb_sum2 = mc.merge(
        [x, y],
        ['betterBlue', 'betterOrange'],
        blending='sum',
        saturation_limits=[(0, 255), (0, 255)],
    )
    assert np.allclose(rgb_sum, rgb_sum2)

def test_apply_colors_and_merge_high_sum():
    '''
    Test that the colors are merged correctly with sum blending and high values
    '''
    x = np.ones((1, 1), dtype=np.uint8) * 255
    y = np.ones((1, 1), dtype=np.uint8) * 255
    rgb_sum = mc.dispatch_multi_channel(
        [x, y],
        ['betterBlue', 'betterOrange'],
        [None, None],
        'sum',
        [(0, 255), (0, 255)],
    )
    # blue = [0, 188, 254]
    # orange = [255, 149, 0]
    assert np.allclose(
        rgb_sum,
        np.array(
            [[[255, 255, 254]]]
        )
    )
    rgb_sum2 = mc.merge(
        [x, y],
        ['betterBlue', 'betterOrange'],
        blending='sum',
        saturation_limits=[(0, 255), (0, 255)],
    )
    assert np.allclose(rgb_sum, rgb_sum2)

def test_apply_colors_and_merge_low_max():
    '''
    Test that the colors are merged correctly with max blending and low values
    '''
    x = np.ones((1, 1), dtype=np.uint8)
    y = np.ones((1, 1), dtype=np.uint8)
    rgb_max = mc.dispatch_multi_channel(
        [x, y],
        ['betterBlue', 'betterOrange'],
        [None, None],
        'max',
        [(0, 255), (0, 255)],
    )
    # blue = [0, 1, 2,]
    # orange = [1, 1, 0]
    assert np.allclose(
        rgb_max,
        np.array(
            [[[1, 1, 2]]]
        )
    )
    rgb_max2 = mc.merge(
        [x, y],
        ['betterBlue', 'betterOrange'],
        blending='max',
        saturation_limits=[(0, 255), (0, 255)],
    )
    assert np.allclose(rgb_max, rgb_max2)

def test_apply_colors_and_merge_high_max():
    '''
    Test that the colors are merged correctly with max blending and high values
    '''
    x = np.ones((1, 1), dtype=np.uint8) * 255
    y = np.ones((1, 1), dtype=np.uint8) * 255
    rgb_max = mc.dispatch_multi_channel(
        [x, y],
        ['betterBlue', 'betterOrange'],
        [None, None],
        'max',
        [(0, 255), (0, 255)],
    )
    # blue = [0, 188, 254]
    # orange = [255, 149, 0]
    assert np.allclose(
        rgb_max,
        np.array(
            [[[255, 188, 254]]]
        )
    )
    rgb_max2 = mc.merge(
        [x, y],
        ['betterBlue', 'betterOrange'],
        blending='max',
        saturation_limits=[(0, 255), (0, 255)],
    )
    assert np.allclose(rgb_max, rgb_max2)

def test_apply_colors_and_merge_low_min():
    '''
    Test that the colors are merged correctly with min blending and low values
    '''
    x = np.ones((1, 1), dtype=np.uint8)
    y = np.ones((1, 1), dtype=np.uint8)
    rgb_min = mc.dispatch_multi_channel(
        [x, y],
        ['betterBlue', 'betterOrange'],
        [None, None],
        'min',
        [(0, 255), (0, 255)],
    )
    # blue = [0, 1, 2,]
    # orange = [1, 1, 0]
    assert np.allclose(
        rgb_min,
        np.array(
            [[[0, 1, 0]]]
        )
    )
    rgb_min2 = mc.merge(
        [x, y],
        ['betterBlue', 'betterOrange'],
        blending='min',
        saturation_limits=[(0, 255), (0, 255)],
    )
    assert np.allclose(rgb_min, rgb_min2)

def test_apply_colors_and_merge_high_min():
    '''
    Test that the colors are merged correctly with min blending and high values
    '''
    x = np.ones((1, 1), dtype=np.uint8) * 255
    y = np.ones((1, 1), dtype=np.uint8) * 255
    rgb_min = mc.dispatch_multi_channel(
        [x, y],
        ['betterBlue', 'betterOrange'],
        [None, None],
        'min',
        [(0, 255), (0, 255)],
    )
    # blue = [0, 188, 254]
    # orange = [255, 149, 0]
    assert np.allclose(
        rgb_min,
        np.array(
            [[[0, 149, 0]]]
        )
    )
    rgb_min2 = mc.merge(
        [x, y],
        ['betterBlue', 'betterOrange'],
        blending='min',
        saturation_limits=[(0, 255), (0, 255)],
    )
    assert np.allclose(rgb_min, rgb_min2)

def test_apply_colors_and_merge_low_mean():
    '''
    Test that the colors are merged correctly with mean blending and low values
    '''
    x = np.ones((1, 1), dtype=np.uint8)
    y = np.ones((1, 1), dtype=np.uint8)
    rgb_mean = mc.dispatch_multi_channel(
        [x, y],
        ['betterBlue', 'betterOrange'],
        [None, None],
        'mean',
        [(0, 255), (0, 255)],
    )
    # blue = [0, 1, 2,]
    # orange = [1, 1, 0]
    assert np.allclose(
        # NOTE: I haven't decided if I'm going to re-normalize after blending so this test may fail
        # in the future
        rgb_mean,
        np.array(
            [[[0, 1, 1]]]
        )
    )
    rgb_mean2 = mc.merge(
        [x, y],
        ['betterBlue', 'betterOrange'],
        blending='mean',
        saturation_limits=[(0, 255), (0, 255)],
    )
    assert np.allclose(rgb_mean, rgb_mean2)

def test_apply_colors_and_merge_high_mean():
    '''
    Test that the colors are merged correctly with mean blending and high values
    '''
    x = np.ones((1, 1), dtype=np.uint8) * 255
    y = np.ones((1, 1), dtype=np.uint8) * 255
    rgb_mean = mc.dispatch_multi_channel(
        [x, y],
        ['betterBlue', 'betterOrange'],
        [None, None],
        'mean',
        [(0, 255), (0, 255)]
    )
    # blue = [0, 188, 254]
    # orange = [255, 149, 0]
    assert np.allclose(
        # NOTE: I haven't decided if I'm going to re-normalize after blending so this test may fail
        # in the future
        rgb_mean,
        np.array(
            [[[127, 168, 127]]]
        )
    )
    rgb_mean2 = mc.merge(
        [x, y],
        ['betterBlue', 'betterOrange'],
        blending='mean',
        saturation_limits=[(0, 255), (0, 255)],
    )
    assert np.allclose(rgb_mean, rgb_mean2)



from __future__ import annotations
from typing import (
	TYPE_CHECKING,
	Sequence,
	Union,
	Tuple,
)

import numpy as np
from nptyping import (
	NDArray,
	Shape,
	UInt8,
)

from mergechannels import (
	dispatch_single_channel,
	dispatch_multi_channel,
)
from ._luts import COLORMAPS
from ._blending import BLENDING_OPTIONS

if TYPE_CHECKING:
	from matplotlib.colors import Colormap as MatplotlibColormap
	from cmap import Colormap as CmapColormap

def _parse_cmap_arguments(
    color: Union[
		COLORMAPS,
		NDArray[Shape['3, 255'], UInt8],
		MatplotlibColormap,
		CmapColormap,
	],
) ->  Tuple[Union[COLORMAPS, None], Union[NDArray[Shape['3, 256'], UInt8], None]]:
	'''
	Parse the color argument and return the corresponding cmap name and cmap values

	Args:
		color: a user-specified argument which may be the name of mergechannels colormap, a ndarray
		lookup table, and matplotlib colormap, or a cmap colormap.

	Returns:
		A tuple specifying the corresponding mergechannels colormap name (or None if N/A), and an
		array of the lookup table (or None if N/A)
	'''
	if isinstance(color, str):
		return color, None
	else:
		try:  # try to convert from a matplotlib colormap
			if not color._isinit:  # type: ignore
				color._init()  # type: ignore
			cmap_values = (color._lut[:color.N, :3] * 255).astype('uint8')  # type: ignore
		except AttributeError:  # try to convert from a cmaps ColorMap
			try:
				cmap_values = (np.asarray(color.lut()[:, :3]) * 255).astype('uint8')  # type: ignore
			except AttributeError:  # must be a list of lists or an array castable to u8 (256, 3)
				cmap_values = np.asarray(color).astype('uint8')  # type: ignore

	if not isinstance(cmap_values, NDArray[Shape['256, 3'], UInt8]):  # type: ignore
		raise ValueError(
			'Expected a matplotlib colormap, a cmaps colormap, or an object directly castable to '
				f'an 8-bit array of shape (256, 3), got {type(cmap_values)}: {color}'
		)
	return None, cmap_values


def apply_color_map(
	arr: np.ndarray,
    color: Union[
		COLORMAPS,
		NDArray[Shape['3, 255'], UInt8],
		MatplotlibColormap,
		CmapColormap,
	],
	percentiles: Union[tuple[float, float], None] = None,
	saturation_limits: Union[tuple[float, float], None] = None,
) -> np.ndarray:
	'''
	apply a color map to an array
	'''
	if saturation_limits is None:
		if percentiles is None:
			percentiles = (1.1, 99.9)
		low, high = np.percentile(arr, percentiles)
		saturation_limits = (low, high)

	cmap_name, cmap_values = _parse_cmap_arguments(color)

	return dispatch_single_channel(
		array_reference=arr,
		cmap_name=cmap_name,
		cmap_values=cmap_values,
		limits=saturation_limits,
	)

def merge(
    arrs: Sequence[np.ndarray],
    colors: Sequence[COLORMAPS],
    blending: BLENDING_OPTIONS = 'max',
    percentiles: Sequence[tuple[float, float]] | None = None,
    saturation_limits: Sequence[tuple[float, float]] | None = None,
) -> np.ndarray:
	'''
	apply cmaps to arrays and blend the colors
	'''
	cmap_names, cmap_values = zip(*[_parse_cmap_arguments(color) for color in colors])
	if saturation_limits is None:
		if percentiles is None:
			percentiles = [(1.1, 99.9)] * len(arrs)
		saturation_limits = tuple(
			np.percentile(arr, ch_percentiles)
			for arr, ch_percentiles in zip(arrs, percentiles)  # type: ignore
		)
	return dispatch_multi_channel(
		array_references=arrs,
		cmap_names=cmap_names,
		cmap_values=cmap_values,
		blending=blending,
		limits=saturation_limits,  # type: ignore
	)



from typing import Literal

COLORMAPS = Literal[
	'16_colors',
	'3-3-2 RGB',
	'3color-BMR',
	'3color-CGY',
	'3color-RMB',
	'3color-YGC',
	'5_ramps',
	'6_shades',
	'BOP blue',
	'BOP orange',
	'BOP purple',
	'Blue',
	'Circus Cherry',
	'Circus Cyan',
	'Circus Green',
	'Circus Ink Black',
	'Circus Ink Cherry',
	'Circus Ink Cyan',
	'Circus Ink Green',
	'Circus Ink Mint',
	'Circus Ink Purple',
	'Circus Ink Yellow',
	'Circus Mint',
	'Circus Purple',
	'Circus Yellow',
	'Cyan',
	'Cyan Hot',
	'Duo Cherry',
	'Duo Cyan',
	'Duo Green',
	'Duo Intense Cherry',
	'Duo Intense Cyan',
	'Duo Intense Green',
	'Duo Intense Mint',
	'Duo Intense Purple',
	'Duo Intense Yellow',
	'Duo Mint',
	'Duo Purple',
	'Duo Yellow',
	'Fire',
	'Grays',
	'Grays g=0.25',
	'Grays g=0.25 inverted',
	'Grays g=0.50',
	'Grays g=0.50 inverted',
	'Grays g=0.75',
	'Grays g=0.75 inverted',
	'Grays g=1.00 inverted',
	'Grays g=1.25',
	'Grays g=1.25 inverted',
	'Grays g=1.50',
	'Grays g=1.50 inverted',
	'Grays g=1.75',
	'Grays g=1.75 inverted',
	'Grays g=2.00',
	'Grays g=2.00 inverted',
	'Green',
	'Green Fire Blue',
	'HiLo',
	'I Blue',
	'I Bordeaux',
	'I Cyan',
	'I Forest',
	'I Green',
	'I Magenta',
	'I Purple',
	'I Red',
	'I Yellow',
	'ICA',
	'ICA2',
	'ICA3',
	'Ice',
	'Ink Black',
	'Ink Cherry',
	'Ink Cyan',
	'Ink Green',
	'Ink Mint',
	'Ink Purple',
	'Ink Wash Black',
	'Ink Wash Cherry',
	'Ink Wash Cyan',
	'Ink Wash Green',
	'Ink Wash Mint',
	'Ink Wash Purple',
	'Ink Wash Yellow',
	'Ink Yellow',
	'Magenta',
	'Magenta Hot',
	'OIMB1',
	'OIMB2',
	'OIMB3',
	'OPF fresh',
	'OPF orange',
	'OPF purple',
	'Orange Hot',
	'Parabolic RGB',
	'Phase Bold Green-Magenta',
	'Phase Bold Ink Green-Magenta',
	'Phase Bold Ink Mint-Cherry',
	'Phase Bold Ink Yellow-Cyan',
	'Phase Bold Mint-Cherry',
	'Phase Bold Yellow-Cyan',
	'Phase Green-Magenta',
	'Phase Ink Green-Magenta',
	'Phase Ink Mint-Cherry',
	'Phase Ink Yellow-Cyan',
	'Phase Mint-Cherry',
	'Phase Yellow-Cyan',
	'Pop blue',
	'Pop blue inverted',
	'Pop cherry',
	'Pop cherry inverted',
	'Pop cyan',
	'Pop cyan inverted',
	'Pop green',
	'Pop green inverted',
	'Pop lavender inverted',
	'Pop magenta',
	'Pop magenta inverted',
	'Pop mint',
	'Pop mint inverted',
	'Pop purple',
	'Pop purple inverted',
	'Pop red',
	'Pop red inverted',
	'Pop yellow',
	'Pop yellow inverted',
	'Quartetto MYGB Blue',
	'Quartetto MYGB Green',
	'Quartetto MYGB Magenta',
	'Quartetto MYGB Yellow',
	'Quartetto RYGB Blue',
	'Quartetto RYGB Green',
	'Quartetto RYGB Red',
	'Quartetto RYGB Yellow',
	'Rainbow RGB',
	'Red',
	'Red Hot',
	'Red/Green',
	'Spectrum',
	'Thermal',
	'Turbo',
	'Yellow',
	'Yellow Hot',
	'betterBlue',
	'betterCyan',
	'betterGreen',
	'betterOrange',
	'betterRed',
	'betterYellow',
	'blue_orange_icb',
	'brgbcmyw',
	'cool',
	'edges',
	'gem',
	'glasbey',
	'glasbey_inverted',
	'glasbey_on_dark',
	'glow',
	'mpl-inferno',
	'mpl-magma',
	'mpl-plasma',
	'mpl-viridis',
	'phase',
	'physics',
	'royal',
	'sepia',
	'smart',
	'thal',
	'thallium',
	'unionjack',
]



from __future__ import annotations
from typing import TYPE_CHECKING, Literal, Sequence, Union
from nptyping import NDArray, Shape, UInt8
import numpy as np

from mergechannels._blending import BLENDING_OPTIONS
from mergechannels._luts import COLORMAPS

if TYPE_CHECKING:
    from matplotlib.colors import Colormap as MatplotlibColormap
    from cmap import Colormap as CmapColormap


Number = Union[int, float]

def apply_color_map(
    arr: np.ndarray,
    color: Union[
        COLORMAPS,
        NDArray[Shape['3, 255'], UInt8],
        MatplotlibColormap,
        CmapColormap,
    ],
    percentiles: tuple[Number, Number] | None = None,
    saturation_limits: tuple[Number, Number] | None = None,
) -> np.ndarray:
    ...

def merge(
    arrs: Sequence[np.ndarray],
    colors: Sequence[COLORMAPS],
    blending: BLENDING_OPTIONS = 'max',
    percentiles: Sequence[tuple[float, float]] | None = None,
    saturation_limits: Sequence[tuple[float, float]] | None = None,
) -> np.ndarray:
    ...

def dispatch_single_channel(
    array_reference: np.ndarray,
    cmap_name: Union[COLORMAPS, None],
    cmap_values: Union[
        NDArray[Shape['3, 255'], UInt8],
        MatplotlibColormap,
        CmapColormap,
        None,
    ],
    limits: tuple[Number, Number] | None = None,
) -> np.ndarray:
    ...

def dispatch_multi_channel(
    array_references: Sequence[np.ndarray],
    cmap_names: Sequence[Union[COLORMAPS, None]],
    cmap_values: Sequence[
        Union[
            NDArray[Shape['3, 255'], UInt8],
            MatplotlibColormap,
            CmapColormap,
            None,
        ]
    ],
    blending: Literal[BLENDING_OPTIONS],
    limits: Sequence[tuple[Number, Number]] | None = None,
) -> np.ndarray:
    ...



# /// script
# requires-python = ">=3.11"
# dependencies = [
#     "numpy",
# ]
# ///

'''
This script converts the luts in the `assets/converted` directory into
1) a lazy static hashmap that with the rgb values for each lut
2) a type hint file for the lut names so that the names are autocompleted in IDEs

This script can be invoked with `uv run scripts/populate_luts.py`
'''

from pathlib import Path
import numpy as np

def open_lut(txt_path: Path) -> np.ndarray:
    '''
    Open the converted lut file
    '''
    assert txt_path.exists(), f'{txt_path} does not exist'
    with open(txt_path, 'r') as f:
        lut = np.loadtxt(f)
    return lut.astype('uint8')

def main():
    '''
    Build the lazy static hashmap and the type hint file for the lut names
    '''
    curr_file = Path(__file__).absolute()
    assets_dir = curr_file.parent.parent / 'assets'
    subdirs = [
        d for d in assets_dir.iterdir()
        if d.is_dir()
        and d.name not in ['converted', 'samples']
        and d.name != 'builtin_luts'
    ]

    lut_data: dict[str, np.ndarray] = {}
    sources = {}
    for subdir in subdirs:
        lut_files = [f.stem for f in subdir.iterdir() if f.is_file() and f.name.endswith('.lut')]
        if subdir.name == 'builtin_luts':
            special_luts = [
                'Fire',
                'Grays',
                'Ice',
                'Spectrum',
                '3-3-2 RGB',
                'Red',
                'Green',
                'Blue',
                'Cyan',
                'Magenta',
                'Yellow',
                'Red%Green',
            ]
            for special_name in special_luts:
                lut_files.append(special_name)
        for lut_file in lut_files:
            txt_path = assets_dir / 'converted' / f'{lut_file}.txt'
            lut = open_lut(txt_path)
            lut_name = txt_path.stem.replace('%', '/')
            lut_data[lut_name] = lut
            sources.setdefault(subdir.name, []).append((lut_name, lut))

    cmaps_file = curr_file.parent.parent / 'src' / 'cmaps.rs'
    assert cmaps_file.exists()

    cmaps_file_lines = ''
    cmaps_file_lines += 'use lazy_static::lazy_static;\n'
    cmaps_file_lines += 'use std::collections::HashMap;\n\n'
    cmaps_file_lines += 'type Colormap = [[u8; 3]; 256];\n\n'
    cmaps_file_lines += 'lazy_static! {\n'
    cmaps_file_lines += "\tpub static ref CMAPS: HashMap<&'static str, Colormap> = {\n"
    cmaps_file_lines += '\t\tlet mut m = HashMap::new();\n'

    for lut_name, lut in sorted(lut_data.items()):
        cmaps_file_lines += f'\t\tm.insert("{lut_name}", [\n'
        for i in range(256):
            cmaps_file_lines += f'\t\t\t[{lut[i, 0]}, {lut[i, 1]}, {lut[i, 2]}],\n'
        cmaps_file_lines += '\t\t]);\n\n'
    cmaps_file_lines += '\t\tm'
    cmaps_file_lines += '\t};'
    cmaps_file_lines += '}'

    with open(cmaps_file, 'w') as f:
        f.write(cmaps_file_lines)

    lut_names_type_hint_file = curr_file.parent.parent / 'python' / 'mergechannels' / '_luts.py'
    lut_names_lines = 'from typing import Literal\n\n'
    lut_names_lines += 'COLORMAPS = Literal[\n'
    for lut_name in sorted(lut_data.keys()):
        lut_names_lines += f"\t'{lut_name}',\n"
    lut_names_lines += ']\n'

    with open(lut_names_type_hint_file, 'w') as f:
        f.write(lut_names_lines)

    readme_path = curr_file.parent.parent / 'README.md'
    assert readme_path.exists()
    readme_lines = readme_path.read_text().splitlines()

    for idx, line in enumerate(readme_lines):
        if line.startswith('## Colormaps'):
            break
    readme_lines = readme_lines[:idx+1]

    lut_mapping = {
        'FIJI built-in LUTs': {
            'source': 'builtin_luts',
            'link': None,
        },
        'My custom LUTs': {
            'source': 'zac_luts',
            'link': 'https://github.com/zacswider/LUTs'
        },
        "Christophe Leterrier's LUTs": {
            'source': 'christ_luts',
            'link': 'https://github.com/cleterrier/ChrisLUTs',
        },
        "James Manton's LUTs": {
            'source': 'jdm_luts',
            'link': 'https://sites.imagej.net/JDM_LUTs/',
        },
    }

    for source_title, source_info in lut_mapping.items():
        source = source_info['source']
        link = source_info['link']
        luts = sources[source]
        if link is not None:
            source_title = f'[{source_title}]({link})'

        readme_lines.append(f'### {source_title}\n')
        for lut_name, _ in sorted(luts, key=lambda x: x[0]):
            lut_path = (
                'https://raw.githubusercontent.com/zacswider/README_Images/main'
                f'/{source}_{lut_name.replace("/", "%25")}.png'
            )
            readme_lines.append(
                (
                    f'<p>{lut_name.replace("%", "/")}: <img src="{lut_path}" '
                    'style="vertical-align: middle"></p>'
                )
            )
        readme_lines.append('')
    readme_path.write_text('\n'.join(readme_lines))


if __name__ == '__main__':
    main()



# Import Rust functions
from .mergechannels import (  # type: ignore
	dispatch_single_channel,
	dispatch_multi_channel,
)
from ._internal import merge, apply_color_map
from ._luts import COLORMAPS as _COLORMAPS
from typing import Literal  # noqa: F401

COLORMAPS = _COLORMAPS


__all__ = [
	'dispatch_single_channel',
	'dispatch_multi_channel',
	'merge',
	'apply_color_map',
	'COLORMAPS',
]
